SystemOrganization addCategory: #'Rest-Client-HTTP'!
SystemOrganization addCategory: #'Rest-Client-Proxies'!

!Dictionary methodsFor: '*rest-client' stamp: 'mt 2/16/2009 10:28'!
keysAndValuesCollect: aBlock 
	| newDict |
	newDict := Dictionary new.
	self keysAndValuesDo: [:key :value | newDict add: (aBlock value: key value: value)].
	^ newDict! !

Object subclass: #RestProxyObject
	instanceVariableNames: 'properties socket baseUri canonicalName factory'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyObject commentStamp: 'mt 3/2/2009 16:14' prior: 0!
I am a proxy object. I store a reference to the real object in form of a url (e.g. http://bp2008h1srv:8082/projects/5/tasks/3). I behave similar to the real object because I understand its accessors.!

!RestProxyObject class methodsFor: 'instance creation' stamp: 'topa 3/3/2009 17:41'!
fromServer: aBaseUriString path: aPathString 

	^ self new 
		canonicalName: aPathString;
		baseUri: aBaseUriString! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:46'!
baseUri

	^ baseUri! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:46'!
baseUri: anObject

	baseUri := anObject.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 16:52'!
canonicalName

	^ canonicalName! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:46'!
canonicalName: anObject

	canonicalName := anObject.! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 2/27/2009 14:13'!
commit

	^ RestSocket
		putDocumentFromUrl: (self baseUri, self canonicalName) asUrl
		request: ''
		content: (Json render: self properties)! !

!RestProxyObject methodsFor: 'updating' stamp: 'cs 2/27/2009 14:52'!
commit: key

	^ RestSocket
		putDocumentFromUrl: (self baseUri, self canonicalName) asUrl
		request: ''
		content: (Json render: (Dictionary with: (key -> (self properties at: key))))! !

!RestProxyObject methodsFor: 'deleting' stamp: 'topa 3/3/2009 16:52'!
delete

	RestSocket
		deleteDocumentFromUrl: (self baseUri, self canonicalName) asUrl
		request: ''.! !

!RestProxyObject methodsFor: 'error handling' stamp: 'topa 3/3/2009 18:08'!
doesNotUnderstand: aMessage

	" handle getters"
	(self properties includesKey: aMessage selector)
		ifTrue: [^ self properties at: aMessage selector].
		
	" handle setters"
	((aMessage selector endsWith: ':') and:
		[self properties includesKey: aMessage selector allButLast])
			ifTrue: [^ self set: aMessage selector allButLast value: aMessage arguments first].

	"handle unknown"
	super doesNotUnderstand: aMessage.! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 18:19'!
factory

	^ factory! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 18:19'!
factory: anObject

	factory := anObject! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 17:40'!
properties

	^ properties ifNil: [self reload. properties]! !

!RestProxyObject methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:47'!
properties: anObject

	properties := anObject.! !

!RestProxyObject methodsFor: 'updating' stamp: 'topa 3/3/2009 21:47'!
reload
	
	| jsonResult |
	jsonResult := Json readFrom: (RestSocket get: (self baseUri, self canonicalName)) content readStream.
	
	self properties: (jsonResult keysAndValuesCollect: [:key :value |
			value isArray
				ifFalse: [key -> value] 
				ifTrue: [key -> (value collect: [:path | self factory getObject: path])]]).
		! !

!RestProxyObject methodsFor: 'accessing' stamp: 'cs 2/27/2009 14:51'!
set: property value: value

	self properties at: property put: value.
	self commit: property.! !

Object subclass: #RestProxyObjectFactory
	instanceVariableNames: 'uri'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-Proxies'!

!RestProxyObjectFactory commentStamp: 'mt 3/2/2009 16:39' prior: 0!
I provide convenient access to a rest server by creating rest proxy objects.!

!RestProxyObjectFactory class methodsFor: 'instance creation' stamp: 'topa 3/3/2009 16:58'!
fromString: aServerString
	"e.g. http://bp2008h1srv:8082/"

	^ self new uri: (SwazooURI fromString: aServerString)! !

!RestProxyObjectFactory methodsFor: 'instance creation' stamp: 'topa 3/3/2009 18:21'!
addObject: path with: anObject 

	| response |
	response := RestSocket 
				postDocumentFromUrl: (self uri asString, path) asUrl
				request: ''
				content: (Json render: anObject).
	^ self getObject: response content! !

!RestProxyObjectFactory methodsFor: 'instance creation' stamp: 'topa 3/3/2009 21:48'!
getCollection: path 
	"Get list of cn's and request their Objects into a Collection"
	
	| jsonCollection |
	jsonCollection :=  Json readFrom: (RestSocket get: self uri asString, path) content readStream.
	^ jsonCollection collect: [:aPath | self getObject: aPath]
! !

!RestProxyObjectFactory methodsFor: 'instance creation' stamp: 'topa 3/3/2009 18:20'!
getObject: path 

	^ (RestProxyObject fromServer: self uri asString path: path)
		factory: self! !

!RestProxyObjectFactory methodsFor: 'accessing' stamp: 'topa 3/3/2009 17:03'!
uri

	^ uri! !

!RestProxyObjectFactory methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:48'!
uri: anObject

	uri := anObject.! !

Object subclass: #RestResponse
	instanceVariableNames: 'content headers responseCode url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestResponse class methodsFor: 'instance creation' stamp: 'mfp 2/20/2009 16:05'!
responseCode: aNumberCode headers: aDictionary content: aString url: aUrl

	^ self new
		responseCode: aNumberCode asNumber;
		headers: aDictionary;
		content: aString;
		url: aUrl! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content

	^ content! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
content: anObject

	content := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:49'!
getHeader: name 

	^ self getHeader: name default: nil! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
getHeader: name  default: defaultValue

	^ self headers at: name  ifAbsent: [defaultValue]! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers

	^ headers! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
headers: anObject

	headers := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode

	^ responseCode! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
responseCode: anObject

	responseCode := anObject.! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:30'!
url

	^ url! !

!RestResponse methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
url: anObject

	url := anObject.! !

Object subclass: #RestSocket
	instanceVariableNames: 'conn headers responseCode request content url'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Rest-Client-HTTP'!

!RestSocket class methodsFor: 'private' stamp: 'topa 3/3/2009 21:40'!
defaultPortForSchema: aSchema
	"default port to connect on"

	aSchema = 'http' ifTrue: [^ 80].
	aSchema = 'https' ifTrue: [^ 443].! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:40'!
deleteDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'DELETE'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 2/25/2009 11:11'!
doHTTPRequestFromUrl: url verb: aVerb request: requestString content: aContentString 
	| serverName sock |
	self flag: #NeedsRefactoring.
	Socket initializeNetwork.

	serverName := url authority.
	3 timesRepeat: [
		sock := self new 
			url: url;
			request: requestString;
			content: aContentString.
		(sock connectAndWaitUntil: (Time millisecondClockValue + 10000 truncated))
			ifTrue: [^ sock doVerb: aVerb]].
	Socket deadServer: serverName.
	sock destroy.
	^ 'Server ' , serverName , ' is not responding'! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
expandUrl: newUrl ip: byteArrayIP port: portNum

	self flag: #SplitUp; flag: #NeedsRefactoring.
	(newUrl beginsWith: '../') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: (newUrl allButFirst: 2)]].
	(newUrl beginsWith: '/') 
		ifTrue: [
			^ String streamContents: [:strm | 
				byteArrayIP do: [:bb | bb printOn: strm.  strm nextPut: $.].
				strm skip: -1; nextPut: $:.
				portNum printOn: strm.
				strm nextPutAll: newUrl]].
	^ newUrl
! !

!RestSocket class methodsFor: 'requesting' stamp: 'SR 2/26/2009 14:59'!
get: aUriString
	"Send an HTTP GET request."

	^ RestSocket 
		getDocumentFromUrl: aUriString asUrl "for Socket compatibility"
		request: ''! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
getDocumentFromUrl: url request: requestString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'GET'
		request: requestString
		content: nil! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
postDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'POST'
		request: requestString
		content: contentString! !

!RestSocket class methodsFor: 'requesting' stamp: 'topa 3/3/2009 21:41'!
putDocumentFromUrl: url request: requestString content: contentString 

	^ self 
		doHTTPRequestFromUrl: url
		verb: 'PUT'
		request: requestString
		content: contentString! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn

	^ conn! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:31'!
conn: anObject

	conn := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
connect
	| serverAddr |
	serverAddr := (NetNameResolver addressForName: self serverName timeout: 20)
		ifNil: [self error: 'Could not resolve the server named: ', self serverName].
		
	self connectTo: serverAddr port: self port.! !

!RestSocket methodsFor: 'api' stamp: 'topa 2/19/2009 19:41'!
connectAndWaitUntil: aTimeout

	^ self connect; waitForConnectionUntil: aTimeout! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
connectTo: addr port: aPort

	self conn: (self usesSSL
			ifTrue: [SSLSocket newConnectionAddress: addr port: aPort version: SSLProtocolVersion tls10]
			ifFalse: [Socket new connectTo: addr port: aPort]).
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content

	^ content! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:32'!
content: anObject

	content := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:32'!
destroy

	self conn close.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:34'!
doVerb: aHTTPVerb
	| cmd stream |
	self flag: #NeedsRefactoring.
	cmd := 	aHTTPVerb,' ', self url fullPath, ' HTTP/1.0', String crlf, 
				'Host: ', self serverName , ':', self port printString, String crlf, 
				(self request ifEmpty: [''] ifNotEmpty: [self request, String crlf]).
					
	self content ifNotNil: [cmd := cmd, 
		'Content-type: application/json', String crlf,
		'Content-length: ', self content size printString , String crlf].
	self sendCommand: cmd.
	"headers are sent."					
	self content ifNotNil: [self sendCommand: self content].
	stream := self getData readStream.
	self headers: (stream upToAll: String crlfcrlf). "parses header, too"
	^ RestResponse 
			responseCode: self responseCode  
			headers: self headers
			content: (self getRestOfData: stream upToEnd)
			url: self url! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:35'!
getData

	| data |
	data := self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveData].
	[self conn isConnected and: [(data asString findString: String crlfcrlf) == 0]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveData])].
	^ data asString! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:36'!
getHeader: name 

	^ self getHeader: name  default: nil! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:36'!
getHeader: name  default: defaultValue

	^ self headers at: name  ifAbsent: [defaultValue]! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:37'!
getRestOfData: begin

	| data length |
	self flag: #NeedsRefactoring.
	length := self getHeader: 'content-length'.
	length ifNotNil: [ length := length asNumber ].
	data := begin, (self usesSSL
		ifTrue: [self conn receive]
		ifFalse: [self conn receiveAvailableData]).
	[self conn isConnected and: [length isNil or: [data size < length]]]
		whileTrue: [
			(Delay forMilliseconds: 100) wait.
				data := data, (self usesSSL
					ifTrue: [self conn receive]
					ifFalse: [self conn receiveAvailableData]).].
	^ data asString! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:37'!
headers

	^ headers! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:37'!
headers: headerText
	"set the headers.  Then getHeader: can be used"

	| lines foldedLines i statusLine |
	self flag: #SplitUp.
	"divide into basic lines"
	lines := headerText findTokens: (String with: Character cr  with: Character linefeed).
	statusLine := lines first.
	lines := lines copyFrom: 2 to: lines size.

	"parse the status (pretty trivial right now)"
	responseCode := (statusLine findTokens: ' ') second.

	"fold lines that start with spaces into the previous line"
	foldedLines := OrderedCollection new.
	lines do: [ :line |
		line first isSeparator ifTrue: [
			foldedLines at: foldedLines size  put: (foldedLines last, line) ]
		ifFalse: [ foldedLines add: line ] ].

	"make a dictionary mapping headers to header contents"
	headers := Dictionary new.
	foldedLines do: [ :line |
		i := line indexOf: $:.
		i > 0 ifTrue: [
			self headers 
			at: (line copyFrom: 1 to: i-1) asLowercase 
			put: (line copyFrom: i+1 to: line size) withBlanksTrimmed ] ].
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
isConnecting

	^ self usesSSL
		ifTrue: [self conn isConnecting]
		ifFalse: [self conn isWaitingForConnection]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
port

	^ self url port 
		ifNil:  [self class defaultPortForSchema: self url schemeName]
		ifNotNil: [self url port]
! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request

	^ request! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:38'!
request: anObject

	request := anObject.! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
responseCode

	^ responseCode! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:38'!
sendCommand: cmd
	
	self usesSSL
		ifTrue: [self conn send: cmd, String crlf]
		ifFalse: [self conn sendData: cmd, String crlf]! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
serverName

	^ self url authority! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url

	^ url! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
url: anObject

	url := anObject.! !

!RestSocket methodsFor: 'accessing' stamp: 'topa 3/3/2009 21:39'!
usesSSL

	^ url schemeName = 'https'! !

!RestSocket methodsFor: 'api' stamp: 'topa 3/3/2009 21:39'!
waitForConnectionUntil: deadline
	"Wait up until the given deadline for a connection to be established. 
	Return true if it is established by the deadline, false if not."

	[self isConnecting and: [Time millisecondClockValue < deadline]]
		whileTrue: [(Delay forMilliseconds: 100) wait].
	^ self conn isConnected
! !
